.login_links link{
    color: @def_green_color;
}

.login_links link:hover{
    color: rgb(255,255,255);
}

.login_labels{
    color: rgb(127,127,127);
    transition: all .115s ease-in-out;
}

.login_version_label{
    color: rgb(127,127,127);
    margin-right: 15px;
}

.default_style_tooltip{
    color: rgb(255,255,255);
    background-color: @overlay_color;
}

.labels_with_info_tooltip_grid{
    background-color: @def_bg_color;
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    outline: none;
    box-shadow: none;
    background-image: none;
    text-shadow: none;
}