.settings_label_grid{
    background-color: rgba(45, 51, 56, 1);
}
.settings_main_logo{
    /* padding: 30px 50px; */
    padding: 50px 30px 0 50px;
}
.settings_main_label{
    /* padding: 20px 50px; */
    padding: 10px 50px 50px;
    margin-left: 45px;
}

/* Settings notebook style*/
notebook.settings_notebook_style{
	outline:none;
}

notebook.settings_notebook_style header{
    -gtk-icon-source: none;
	outline:none;
	padding: 0;
    background-color: transparent;
    border: none;
}

notebook.settings_notebook_style tabs{
	outline:none;
	padding: 0;
	margin: 0;
}
notebook.settings_notebook_style tab{
    box-shadow: none;
    border: transparent;
    background-image: none;
	outline:none;
	padding: 0;
    margin: 0;
    transition: all .115s ease-in-out;
}

.default_notebook_grid_content{
    color: rgb(255,255,255);
    background-color: @def_bg_color;
    padding: 30px 100px;
    padding-bottom: 200px;
}

.default_settings_tab_conf{
    padding: 15px 40px;
    transition: all .115s ease-in-out;
}

.split_tunnling_list{
    background-color: @overlay_color;
    color: rgb(255,255,255);
    padding: 10px;
    /* border-radius: 5px; */
}
.split_tunnling_list text{
    background-color: @overlay_color;
    color: rgb(255,255,255);
    /* border-radius: 5px; */
}
