overshoot, undershoot{
    background-image: none;
    border: none;
}

headerbar box button{
	background-image: none;
	background: none;
	box-shadow: none;
	text-shadow: none;
	color: rgb(128, 128, 128);
	border-radius: 1px;
	padding: 5px;
}

headerbar box button:hover{
	background-color: @def_bg_color;
	outline-width: 100px;
	color: rgb(255,255,255);
}


headerbar{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    -gtk-icon-shadow: none;
    outline: none;
    box-shadow: none;
    background-image: none;
    text-shadow: none;
    border: none;
    background-color: @overlay_color;
}

headerbar box button {
    text-shadow: none;
    -gtk-icon-shadow: none;
    color: rgb(255,255,255)
}

menubar{
    border: none;
    box-shadow: none;
    background-color: @overlay_color;
    color: rgb(255,255,255);
}

menubar menu menuitem{
    color: rgb(255,255,255);
    margin-left: -20px;
}

menubar menu menuitem:hover{
    background-color: @def_green_color;
}

menubar menuitem window menu{
    border-radius: 0;
    box-shadow: none;
    border:none;
    color: rgb(255,255,255);
}

.top_menu_hamburger{
    margin-top: 2px;
    margin-bottom: 2px;
}